﻿<#
.SYNOPSIS
    Installs SwyxWare Core HealthMonitoring Service
.DESCRIPTION
    Installs SwyxWare Core Health Monitoring Service. Existing service will be removed and re-installed
.PARAMETER ServiceName
    Name of the service
.PARAMETER ServiceDisplayName
    Display name of the service
.PARAMETER Description
    Description of the service
.PARAMETER StartupType
    Service Startup Type
.PARAMETER ListeningUrls
    URLs the service is listening on. If you want to bind to all IPs use something like "http://*.5000"
.PARAMETER StartAfterInstallation
    Start service after installation
.PARAMETER AdaptRestartOptions
    Adapts service restart options to let Windows restart monitored services only once.
.PARAMETER EnableHyperVSupport
    Enable Hyper-V Support. Healthmonitor will be configured to have local administrative rights which is 
    required by Microsoft to use the Hyper-V heartbeat API.
.PARAMETER AutomaticProbeConfiguration
    Automatically adapts appSettings.json with probing for all installed SwyxWare Services. If set to $False
    appSettings.json Probe settings must be configured manually
.PARAMETER IPAddress
    IP address to use for probe configuration. This parameter is ignored when AutomaticProbeConfiguration is $false.
    When this parameter is not set, the script tries to determine the IP address automatically.
#>
[CmdletBinding(SupportsShouldProcess = $True)]
param($ServiceName = "Swyx.Core.HealthMonitor",
    $ServiceDisplayName = "SwyxWare Core HealthMonitor Service",
    $Description = "Monitors SwyxWare Core servies and provides prometheus and VMWare Heartbeat API",
    $StartupType = "AutomaticDelayedStart",
    $ListeningUrls = "http://localhost:5000",
    [Switch]$StartAfterInstallation = $false,
    [Switch]$AdaptRestartOptions = $True,
    [Switch]$EnableHyperVSupport = $false,
    [Switch]$AutomaticProbeConfiguration = $true,
    $IPAddress)

Set-StrictMode -Version Latest

if (!(New-Object Security.Principal.WindowsPrincipal([Security.Principal.WindowsIdentity]::GetCurrent())).IsInRole([Security.Principal.WindowsBuiltInRole]::Administrator)) {
    Write-Error "You need to run this script elevated."
}

$AppSettingsFile = Join-Path (Split-Path $PSCommandPath -Parent) ".\appsettings.json"

    # Services supported by this script
$Services = @{
    "ConferenceMgr" = 5062;
    "IpPbx.uaCSTA"  = 65012;
    "IpPbx3pcc"     = 65009;
    "IpPbxCDS"      = 9100;
    "IpPbxFaxSrv"   = 61000;
    "IpPbxGate"     = 5064;
    "IpPbxSrv"      = 5060;
    "LinkMgr"       = 65001;
    "PhoneMgr"      = 65007;
    "IpPbxPNSrv"    = 5096;
}


function Grant-ServiceAccess {
    param($Folder,
        $ServiceName)

    Write-Verbose "Grant-Service $ServiceName access permission on $Folder"

    # We cannot use get-acl here, because that returns an ACL which would cause Set-Acl (see below) to set the owner 
    # which is not allowed in all cases. We just want to add an ACE
    $Acl = (Get-Item -Path $Folder).GetAccessControl([System.Security.AccessControl.AccessControlSections]::Access)
    $aclRuleArgs = "NT Service\$ServiceName", "Modify", "ContainerInherit,ObjectInherit", "None", "Allow"

    $accessRule = New-Object System.Security.AccessControl.FileSystemAccessRule($aclRuleArgs)
    $Acl.AddAccessRule($accessRule)
    $Acl | Set-Acl -Path $Folder
}


function Install-SwyxCoreHealthMonitorService {
    [CmdletBinding(SupportsShouldProcess = $true)]
    param($ServiceName = "Swyx.Core.HealthMonitor",
        $ServiceDisplayName = "SwyxWare Core HealthMonitor Service",
        $Description = "Monitors SwyxWare Core servies and provides prometheus and VMWare Heartbeat API",
        $StartupType = "AutomaticDelayedStart",
        $ListeningUrls)

    if ($PSCmdlet.ShouldProcess($ServiceName, "(Re-)Install Service")) {
        $ServiceExeFolder = Split-Path $PSCommandPath -Parent

        $ServiceCommandLine = (Join-Path $ServiceExeFolder "Swyx.Core.HealthMonitor.exe") + " --service --urls `"$ListeningUrls`""

        $CompatibleStartupType = $StartupType

        if ($StartupType -eq "AutomaticDelayedStart") {
            # workaround for PS 5 compatibility
            $CompatibleStartupType = "Automatic"
        }

        # delete existing service to make sure we end up with a properly configured one
        Write-Verbose -Message "Stop and delete $Servicename if it already exists"
        Stop-Service -ServiceName $ServiceName -Force -ErrorAction SilentlyContinue
        sc.exe delete $ServiceName | Out-Null
    
        # Note: We have to set a password for the credential object to be created. It
        # will be ignored for the LOCALSERVICE account by new-service
        $ServiceCredential = New-Object "PSCredential" -ArgumentList "NT Authority\LocalService", (New-Object "SecureString")

        Write-Verbose "Install-Service $ServiceName"
        New-Service -Name $ServiceName `
            -DisplayName $ServiceDisplayName `
            -Description $Description `
            -StartupType $CompatibleStartupType `
            -BinaryPathName $ServiceCommandLine `
            -Credential $ServiceCredential | Out-Null

        if ($StartupType -eq "AutomaticDelayedStart") {
            sc.exe config $ServiceName start= delayed-auto | Out-Null
        }

        Write-Verbose "Set unrestricted service sid NT Service\$ServiceName"
        sc.exe sidtype $ServiceName unrestricted | Out-Null

        Grant-ServiceAccess -Folder $ServiceExeFolder -ServiceName $ServiceName
    }
}


function Set-TraceFolderPermission {
    [CmdletBinding(SupportsShouldProcess = $true)]
    param($AppSettings, $ServiceName)

    $TraceFile = $AppSettings.Serilog.WriteTo |
    Where-Object { $_.Name -eq "File" } |
    Select-Object -ExpandProperty Args |
    Select-Object -ExpandProperty path

    if ($TraceFile) {
        $TracePath = Split-Path -Path $TraceFile -Parent
        Write-Verbose "TraceFolder Path from appsettings.json: $TracePath"

        if ($PSCmdlet.ShouldProcess($TracePath, "Grant $ServiceName write permission")) {
            if (!(Test-Path $TracePath)) {
                mkdir -Path $TracePath -Force | Out-Null
            }
            Grant-ServiceAccess -Folder $TracePath -ServiceName $ServiceName
        }
    }
}


function Install-EventLogMessageSource {
    <#
    .SYNPOSIS
        Install event log message Source
    .DEASCRIPTION
        Creates an event log message source for the service
        Note: This is also a workaround for dotnet core not having a default message dll. We configure the .Net Framework default message dll
    #>
    [CmdletBinding(SupportsShouldProcess = $true)]
    param($EventLogSourceName,
        $LogName)

    if (![System.Diagnostics.EventLog]::SourceExists($EventLogSourceName) -and `
            $PSCmdlet.ShouldProcess("Swyx.Core.HealthMonitor", "Create Eventlog source in SwyxWare EventLog")) {
        Write-Verbose "Create Event Source $EventLogSourceName in eventlog $LogName"
        [System.Diagnostics.EventLog]::CreateEventSource($EventLogSourceName, $LogName)
    }
}

function Get-MonitoredServices {
    param($AppSettings, $Services)

    $AppSettings.Probes | ForEach-Object {
        $Uri = New-Object "System.Uri" -ArgumentList ([string]($_.Configuration.destination))
       
        $Services.GetEnumerator() | 
        Where-Object { $_.Value -eq $Uri.Port } |
        Select-Object -ExpandProperty Key
    }
}

function Set-IpPbxServiceRestartOptions {
    param($AppSettings)

    Get-MonitoredServices -AppSettings $AppSettings -Services $Services | ForEach-Object {
        sc.exe failure $_ reset=86400 actions= "restart/0//" | Out-Null
    }
}

function Get-AppSettings {
   
    # This somewhat convoluted reading & parsing is necessary for Powershell 4.0 support (Windows Server 2012 R2)
    # It removes comments (which are supported in JSON parsing in PS 6 only) and ensures to pass one string to
    # ConvertFrom-Json for PS 4 compatibility
    $TempFileContent = ""
    (Get-Content $Script:AppSettingsFile) -replace '^\s*//.*' | 
    ForEach-Object { } { $TempFileContent += " $_" } { $TempFileContent } |
    ConvertFrom-Json
}

function Format-Json([Parameter(Mandatory, ValueFromPipeline)][String] $json) {
    $indent = 0;
    ($json -Split "`n" | % {
        if ($_ -match '[\}\]]\s*,?\s*$') {
            # This line ends with ] or }, decrement the indentation level
            $indent--
        }
        $line = ('  ' * $indent) + $($_.TrimStart() -replace '":  (["{[])', '": $1' -replace ':  ', ': ')
        if ($_ -match '[\{\[]\s*$') {
            # This line ends with [ or {, increment the indentation level
            $indent++
        }
        $line
    }) -Join "`n"
}

function Set-AppSettings {
    param([parameter(ValueFromPipeline=$True)]$AppSettings)
	
    Copy-Item -LiteralPath $Script:AppSettingsFile -Destination ($AppSettingsFile + (get-date -Format "-bak-yyyyMMdd-hhmmss"))
    ConvertTo-Json -InputObject $AppSettings -Depth 100 |
        Format-Json | 
        Set-Content -LiteralPath $Script:AppSettingsFile -Force
}

function Enable-HyperVSupport
{
    param($ServiceName)

    Add-ServiceAccountToAdminGroup -ServiceName $ServiceName

    $AppSettings = Get-AppSettings

    $AppSettings.HealthMonitorApp.HyperVHeartbeat = $True
    $AppSettings.HealthMonitorApp.VSphereHeartbeat = $False

    $appSettings | Set-AppSettings
}

function Add-ServiceAccountToAdminGroup
{
    param($ServiceName)

    # we cannot use Add-LocalGroupMember here, because that's not available on Windows Server 2012 R2
    # net.exe localgroup does not work either, as it does not support names longer than 20 characters

    [System.Reflection.Assembly]::LoadWithPartialName("System.DirectoryServices.AccountManagement") | Out-Null
     
    $ServiceAccountName = "NT Service\$ServiceName"
   
    $PrincipalContext = new-object "System.DirectoryServices.AccountManagement.PrincipalContext" -ArgumentList "Machine"
    $AdminGroup = [System.DirectoryServices.AccountManagement.GroupPrincipal]::FindByIdentity($PrincipalContext, 'S-1-5-32-544')
    $IsMember = [bool]($AdminGroup.GetMembers() | Where-Object { $ServiceAccountName -match $_.Name })

    if (!$IsMember) {
        $ServiceAccount = [System.Security.Principal.NTAccount]::New($ServiceAccountName)
        $ServiceSID = $ServiceAccount.Translate([System.Security.Principal.SecurityIdentifier])
        $ServiceAccountPrincipal = [System.DirectoryServices.AccountManagement.GroupPrincipal]::FindByIdentity($PrincipalContext, $ServiceSID)
        Write-Verbose "Add $($ServiceAccountPrincipal.Name) to $($AdminGroup.Name) group"
        $AdminGroup.Members.Add($ServiceAccountPrincipal);
        $AdminGroup.Save();
	}
    else {
        Write-Verbose "$($ServiceAccountName) already member of $($AdminGroup.Name) group"
	}
}

function Get-SwyxWareIpAddress
{
    if ($Script:IPAddress)
    {
        $IpAddress = $Script:IPAddress
    }
   
    if (!$IpAddress) {
        # If IP has been explicitly set in registry, use that one
        $IpAddress = "T-Com","Swyx" | ForEach-Object {
            $IpAddressItem = Get-ItemProperty -path "HKLM:\SOFTWARE\WOW6432Node\$_\General\CurrentVersion\Options\" -Name "LocalIpAddress" -ErrorAction SilentlyContinue
            if ($IpAddressItem) {
                $IpAddressItem.LocalIpAddress
            }
        } | Select-Object -First 1
    }
    
    if (!$IpAddress) {
        $IpAddress = (Get-NetIPConfiguration |
            Where-Object {
                $_.IPv4DefaultGateway -ne $null -and
                $_.NetAdapter.Status -ne "Disconnected"
        }
        ).IPv4Address.IPAddress
    }
    
    if (!$IpAddress) {
        throw "Could not determine IP address. Cannot continue"
	}

    $IpAddress
}


function Remove-ProbeForService {
    param($AppSettings, $ServicePort)

    $ResultingProbes = $AppSettings.Probes | Where-Object {
        ([Uri]$_.Configuration.Destination).Port -ne $ServicePort
	}

    if (!$ResultingProbes) { $appSettings.Probes = @() } else { $AppSettings.Probes = $ResultingProbes }

    $AppSettings
}

function Set-AppSettingsProbeConfiguration
{
    $IpAddress = Get-SwyxWareIpAddress

    $AppSettings = Get-AppSettings 

    $AppSettings.Probes | ForEach-Object {
        Write-Verbose -Message "Configuring probe $($_.Name) with IP address $IpAddress"
        $_.Configuration.Destination = $_.Configuration.Destination -replace "(.+:)//(.+)(:.+)","`$1//$IpAddress`$3"
	}

    $Services.GetEnumerator() | Foreach-Object {
        if (!(Get-Service -Name $_.Name -ErrorAction SilentlyContinue)) {
            $AppSettings = Remove-ProbeForService -AppSettings $AppSettings -ServicePort $_.Value
		}
	}

    $AppSettings | Set-AppSettings

    $Script:AppSetting = $AppSettings
}


#
# Execution starts here
#

$AppSettings = Get-AppSettings

Install-SwyxCoreHealthMonitorService -ServiceName $ServiceName `
    -ServiceDisplayName $ServiceDisplayName `
    -Description $Description `
    -StartupType $StartupType `
    -ListeningUrls $ListeningUrls

Set-TraceFolderPermission -AppSettings $AppSettings -ServiceName $ServiceName

Install-EventLogMessageSource -EventLogSourceName $ServiceName `
    -LogName "SwyxWare"

if ($EnableHyperVSupport -and $PSCmdlet.ShouldProcess("HealthMonitor Service Account", "Add to Administrators group")) {
    Enable-HyperVSupport -ServiceName $ServiceName
}

if ($AdaptRestartOptions -and $PSCmdlet.ShouldProcess("Monitored Services", "Adapt restart options")) {
    Set-IpPbxServiceRestartOptions -AppSettings $AppSettings
}

if ($AutomaticProbeConfiguration -and $PSCmdlet.ShouldProcess("appSettings.json", "auto-configure probes")) {
    Set-AppSettingsProbeConfiguration
}

if ($StartAfterInstallation -and $PSCmdlet.ShouldProcess($ServiceName, "Start service")) {
    Start-Service -ServiceName $ServiceName
} 
