# Swyx.Core.HealthMonitor

A service for monitoring SwyxWare core services and providing

- a VMWare heartbeat (VMWare Guest SDK API)
- a Hyper-V heartbeat (Hyper-V Heartbeat Integration service API)
- a Prometheus metrics endpoint to provide health status

## Getting Started

The service is an asp.net core service which usually runs as Windows service. It can also be started
as a stand-alone console app. The latter is intended for debugging and testing only.

### Installing as a Windows Service

1. Unzip all files into a new, empty folder
2. Open an elevated PowerShell prompt in the folder created in step 1
3. Run `install-service.ps1`.  Per default the service only listens on localhost. To make the prometheus API available from other systems, use the -ListeningUrls parameter, e.g. `-ListeningUrls 'http://*:5000'`. For hyper-V support add the -EnableHyperVSupport parameter. The script offers some more parameters. Use `get-help ./install-service.ps1 -full` for details.
4. The service will be configured to start automatically. Execute ```Start-Service Swyx.Core.HealthMonitor``` to start it manually after installation.

### Settings

The service is configured via appSettings.json. For development and test place your configuration in appSettings.Development.json.

Example settings file:

```json
{
  "HealthMonitorApp": {
    "Prometheus": true,
    "VSphereHeartbeat": false,
    "HyperVHeartbeat": false
  },
  "Probes": [
    {
      "Name": "swyx_server",
      "Type": "Network",
      "Configuration": {
        "destination": "sip://example.com:5060",
        "probingIntervalSeconds": 10,
        "responseTimeoutMilliseconds": 5000
      }
    },
    {
      "Name": "cds",
      "Type": "ConfigDataStore",
      "Configuration": {
        "destination": "https://example.com:9100/ippbx/client/v1.0/login/systemhealth",
        "skipTlsCheck": false,
        "probingIntervalSeconds": 10,
        "responseTimeoutMilliseconds": 500
      }
    }
    ],
    "HealthRules": [
      // ...
    ],
    "Serilog": {
        // ...
    }
}

````

#### HealthMonitorApp

The `HealthMonitorApp` section has three members to enable the prometheus API, the VMWare vSphere heartbeat and the Hyper-V heartbeat API. If you enable one of the heartbeat APIs without running in a VM where the API is available, health monitor will not start.

#### Probes

The `probes` list contains one or more probes defining which services to monitor. Each probe must have a unique name.

The following probe types are supported
| Type | Description |
|------|-------------|
| Network | Probes via SIP or http and measures response time in milliseconds
| ConfigDataStore | Probes ConfigDataStore system health REST API and generates some metrics from it. HealthMonitor's evaluator includes them in overall health evaluation

Each probe has several configuration items
| key | value |
|-----|--------|
| destination | A destination url. Has to be a valid uri, e.g. `sip://host:port` for SIP probes, or `https://host/path` for http probes. |
|probingIntervalSeconds| Defines the interval probe requests are sent. |
|responseTimeoutMilliseconds| Defines how long to wait for a response. |
|||

Probes of type ```SIP``` can have these additional configuration items
| key | value |
|-----|--------|
| sipTransportMode | SIP transport mode. Can be ```"udp"``` or ```"tcp"```. ```"udp"``` is the default |
|||

Probes of type ```ConfigDataStore``` can have these additional configuration items
| key | value |
|-----|--------|
| skipTlsCheck | Defines if a http probe will skip the server TLS certificate check. Set to ```true``` if you probe an http endpoint with a self-signed or otherwise invalid certificate. When set to ```false``` which is the default value, probing will fail if server presents an invalid certificate |
|||

#### Health Rules

The rules Health Monitor uses to determine overall system health are also defined in the  ```appSettings.json``` file. The system is considered healthy if all rules are satisfied. The following rules are supported

##### Numeric

Checks a numeric metric against a threshold. Example:

``` json
{
  "Type": "Numeric",
  "ProbeName": "server",
  "MetricName": "swyx_number_of_failed_probes",
  "Operation": "LessThanOrEqual",
  "Threshold": 3
}
```

The above rule checks the metric ```swyx_number_of_failed_probes```. If the value is less than or equal 3 the rule is satisfied.

##### Boolean

Checks a boolean metric against a given value. Example:

``` json
{
  "Type": "Boolean",
  "ProbeName": "cds",
  "ProbeTarget": "ippbxsrv",
  "MetricName": "swyx_cds_client_initialised",
  "Value": true,
  "GracePeriodSeconds": 120
}
```

The above rule is satisfied if the metric ```swyx_cds_client_initialised``` delivered by probe ```cds``` for service (```ProbeTarget```) ```ippbxsrv``` is true.

The ```ProbeTarget``` property can be used for probes which return metrics for more than one service. The ConfigDataStore probe currently is the only probe type which does this. Probes not supporting this always return metrics where the ```ProbeTarget``` is identical to the ```ProbeName```.

##### Health Rule parameter

All Health rules support the following parameter:

| Parameter | Description | Default value |
|------|------|------|
|Type| Rule type. Allowed values are ```Numeric``` or ```Boolean``` | None. Parameter is mandatory. |
|MetricName|Name of the metric the rule evaluates| None. Parameter is mandatory. |
|Probename | Name of the probe the metric to evaluate is coming from | None. Parameter is mandatory. |
|ProbeTarget| For probe which reports metric from multiple targets this parameter defines the target the metric is coming from. | If not set rule applies to all metrics defined with ```MetricName``` coming from the probe |
|GracePeriod| Time in second the rule is allowed to evaluate to false before health monitor considers it begin false.| 0 = no grace period.|
|||

```Numeric``` health rules additionally support the following parameter
| Parameter | Description | Default value |
|------|------|------|
|Threshold|Threshold value to compare the metric against| None. Parameter is mandatory. |
|Operation|Comparison operation used by the rule. Supported values</br>- LessThan</br>- LessThanOrEqual</br>- GreaterThanOrEqual</br>- GreaterThan| None. Parameter is mandatory. |
|||

```Boolean``` health rules additionally support the following parameter
| Parameter | Description | Default value |
|------|------|------|
|Value|Value to compare the metric against. Can be ```true``` or ```false```| None. Parameter is mandatory. |
|||

#### Metrics

Health monitor generates the following metrics

| Metric name | Description |
|------|------|
|swyx_system_health| 1 if SwyxWare is healthy, 0 otherwise |
| swyx_response_time_milliseconds | Response time of last probing in milliseconds. The service being probed is reported via label ```service``` |
| swyx_number_of_failed_probes | Number of consecutive failed probes of the service. The service failed to respond is reported via label ```service``` |
| swyx_database_schema_match | 1 if actual database schema matches CDS expectation or 0 otherwise. |
| swyx_last_server_ping_seconds_ago | Time in seconds since SwyxServer contacted CDS |
| swyx_server_seconds_since_last_call |Time in seconds since SwyxServer's last call (reported from CDS)|
|swyx_cds_client_initialised| 1 if CDS client reported successful initialisation to CDS, 0 otherwise. The label ```probeTarget``` indicates the service reporting the state. At the time of this writing ```ippbxsrv```, ```phonemgr``` and ```ippbxuacsta``` report their CDS client initialisation state to CDS|
|swyx_system_license_type|SwyxWare license type 0=Enterprise, 1=Hosted, 2=Cloud, 3=CloudLicensed|
|swyx_system_is_licensed| 1 if SwyxWare is licensed, 0 otherwise |
|||

#### Serilog

For configuring the logging framework see: [Serilog.Settings.Configuration](https://github.com/serilog/serilog-settings-configuration)

## Logging

Per default the service creates one log file per day in ```c:\ProgramData\Swyx\traces```. Windows-service-specific events are logged in the SwyxWare event log.

## Grafana/Prometheus Sample Configuration

The ```Monitoring``` folder contains sample configurations for a [Prometheus](https://prometheus.io) server to collect health monitor metrics and some [Grafana](https://grafana.com) dashboards to be used as a starting point for visualising the data. The sample configuration files have been tested with Prometheus 2.16.0 and Grafana 6.6.1. For an easy setup of both services a docker-compose.yml file is provided as well. If you have docker and docker-compose installed use ```docker-compose up -d``` to start a prometheus and grafana service. Full installation instructions can be found in the [SwyxWare documentation](https://help.swyx.com).
